@@ -56,7 +56,7 @@
 
 
 
-focalMat <- function(x, d, type=c('circle', 'Gauss', 'rectangle'), fillNA=FALSE) {
+focalMat <- function(x, d, type=c("circle", "Gauss", "rectangle"), fillNA=FALSE) {
 	type <- match.arg(type)
 	x <- res(x)
 	if (type == 'circle') {
