@@ -378,7 +378,7 @@ setMethod("nearest", signature(x="SpatVector"),
 			from <- zz[seq(1, nrow(zz), 2), ]
 			to <- zz[seq(2, nrow(zz), 2), ]
 			values(to) <- data.frame(id=1:nrow(to))
-			to_int <- as.data.frame(intersect(to, y))
+			to_int <- as.data.frame(intersect(to, y)[1])
 			to_int <- as.data.frame(merge(to, to_int, by="id", all.x=TRUE))
 			if (any(is.na(to_int$to_id))) {
 				zz <- as.points(elongate(z, 1))
