@@ -75,7 +75,7 @@ get_filetype <- function(filename) {
 	fn <- tolower(filename)
 	ext <- tools::file_ext(fn)
 	ext2 <- gsub("^[^.]*\\.(.*).*$", "\\1", fn)
-	if (ext == "shp" || ext == "shz" || (ext == "zip" && ext2 == "shp.zip") || ext == "") {
+	if (ext == "shp" || ext == "shz" || (ext == "zip" && ext2 == "shp.zip")) {
 		"ESRI Shapefile"
 	} else if (ext == "gpkg" || (ext == "zip" && ext2 == "gpkg.zip")) {
 		"GPKG"
